"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const PARAM_REG = /%(?:([1-9]\d*)\$|\(([^\)]+)\))?(?:\+)?(?:0|'[^$])?(?:-)?(?:\d+)?(?:\.(?:\d+))?([bcdieufgostTvxXj])/g;
function parseParams(format) {
    const formatParams = getFormatParams(format);
    if (!formatParams.length) {
        return [];
    }
    const params = formatParamsToTsParams(formatParams);
    return params;
}
exports.parseParams = parseParams;
function formatParamsToTsParams(formatParams) {
    const list = [];
    const formatParamsWithNames = [];
    let countParamsWithOrder = 0;
    formatParams.forEach((param, i) => {
        if (param.name) {
            formatParamsWithNames.push(param);
            return;
        }
        let index = i - countParamsWithOrder - formatParamsWithNames.length;
        if (param.order) {
            countParamsWithOrder++;
            index = param.order - 1;
        }
        if (list[index]) {
            return;
        }
        const type = getParamTypes(param.type);
        const name = 'p' + (index + 1);
        list[index] = { name, type };
    });
    if (formatParamsWithNames.length) {
        const type = [formatDataParamType(formatParamsWithNames)];
        const name = 'p' + (list.length + 1);
        list.push({ name, type });
    }
    return list;
}
function formatDataParamType(formatParams) {
    const data = {};
    formatParams.forEach((param) => {
        const name = param.name || '';
        if (!data[name]) {
            data[name] = getParamTypes(param.type);
        }
    });
    return '{ ' + Object.keys(data).map(item => item + ': ' + data[item].join(' | ')).join('; ') + ' }';
}
function getFormatParams(format) {
    const list = [];
    let result;
    while ((result = PARAM_REG.exec(format)) !== null) {
        if (result.length === 4) {
            list.push({
                order: parseInt(result[1]),
                name: result[2],
                type: result[3],
            });
        }
        else if (result.length === 3) {
            if (/^\d+$/.test(result[1])) {
                list.push({
                    order: parseInt(result[1]),
                    type: result[2],
                });
            }
            else {
                list.push({
                    name: result[1],
                    type: result[2],
                });
            }
        }
        else {
            list.push({
                type: result[1],
            });
        }
    }
    return list;
}
function getParamTypes(formatType) {
    switch (formatType) {
        case 'b':
        case 'c':
        case 'd':
        case 'i':
        case 'e':
        case 'u':
        case 'f':
        case 'g':
        case 'o':
        case 'x':
        case 'X':
            return ['number'];
        case 's':
            return ['string'];
        case 't':
            return ['boolean'];
        case 'T':
        case 'v':
            return ['any'];
        case 'j':
            return ['object', 'any[]'];
    }
    throw new Error(`Invalid format type ${formatType}`);
}
